@@ -6,7 +6,7 @@
  * forth in the LICENSE file which can be found at the top level of
  * the sendmail distribution.
  *
- *	$Id: sm_os_osf1.h,v 1.2 2001/10/01 14:10:45 ca Exp $
+ *	$Id: sm_os_osf1.h,v 1.3 2001/10/09 23:12:13 ca Exp $
  */
 
 /*
@@ -16,8 +16,3 @@
 #define SM_OS_NAME "osf1"
 
 #define SM_CONF_SETITIMER 0
-
-/* can't set real gid */
-#ifndef SM_CONF_CANT_SETRGID
-# define SM_CONF_CANT_SETRGID	1
-#endif /* SM_CONF_CANT_SETRGID */
